<?php

/**
 * Copyright (c) 2021 Kount, Inc.

 * This file is part of Kount Fraud Prevention.

 * Kount Fraud Prevention is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Kount Fraud Prevention is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Kount Fraud Prevention.  If not, see <https://www.gnu.org/licenses/>.
 */

/**
 * KFPWOO_Pre_Auth
 *
 * This is used to class to collect pre auth data and sent to the Kount Server
 *
 * @since      1.0.0
 * @package    Kount
 * @subpackage Kount/Services
 * @author     Kount Inc. <developer@kount.com>
 *
 */

class KFPWOO_Pre_Auth
{

    /**public variables */
    public $order, $payment_type, $event_logging, $current_session_id,
        $call_api_obj, $kount_config, $constants_text, $auth;

    /**
     * __construct
     * It will call automatically when class object created
     * @return void
     */
    public function __construct()
    {
        /**configuration */
        $config                     =   new KFPWOO_Config_;
        $this->kount_config         =   $config->config_();
        /**Constants */
        $costants_obj               =   new KFPWOO_Constants_();
        $this->constants_text       =   $costants_obj->kfpwoo_constants_text();
        $this->payment_type         =   $costants_obj->kfpwoo_payment_type();
        //logs class object (responsible for logs)
        $this->event_logging        =   new KFPWOO_Event_logging();
        //current session id
        $this->current_session_id   =   sanitize_text_field($_SESSION['KFPWOO_SESSION_ID']);
        //Http call
        $this->call_api_obj         =   new KFPWOO_Call_API();
    }

    /**
     * kfpwoo_get_checkout_page_form_data
     * Getting checkout page form data
     * @return void
     */
    public function kfpwoo_get_checkout_page_form_data($order_id)
    {
        $this->kfpwoo_set_order($order_id);
        $this->kfpwoo_create_json_payload();
    }

    /**
     * kfpwoo_set_order
     * Get all order details and set in $order variable
     * @param  mixed $order_id
     * @return void
     */
    function kfpwoo_set_order($order_id)
    {
        $this->order = wc_get_order($order_id);
    }

    /**
     * kfpwoo_get_product_details
     * Get all cart product details
     * @return void
     */
    function kfpwoo_get_product_details()
    {
        $shopping_cart_items  = [];
        $line_item_properties = [];
        $this->discount = 0;
        $pf = new WC_Product_Factory();
        foreach ($this->order->get_items() as $order_items => $order_item) {
            $product = $order_item->get_product();
            if ($product->get_sale_price() !== '') {
                $this->discount = $this->discount + ($product->get_regular_price() - $product->get_sale_price()) * $order_item->get_quantity();
            }

            $item = array(
                $this->constants_text['SKU']          => $product->get_sku(),
                $this->constants_text['PRODUCT_ID']   => strval($order_item->get_product_id()),
                $this->constants_text['NAME']         => $order_item->get_name(),
                $this->constants_text['PRICE']        => strval($order_item->get_subtotal() / $order_item->get_quantity()),
                $this->constants_text['QUANTITY']     => $order_item->get_quantity(),
                $this->constants_text['IS_DIGITAL']   => $pf->get_product($order_item->get_product_id())->is_downloadable('yes'),
                $this->constants_text['IS_GIFT_CARD'] => false
            );

            /***get product category */
            $product_cat_name = '';
            $terms = get_the_terms($order_item->get_product_id(), 'product_cat');
            if ($terms) {
                foreach ($terms  as $term) {

                    $product_cat_name = $product_cat_name . " " . $term->name;
                }
                $category = substr($product_cat_name, 0, 255);

                if ($category != "") {
                    $product_type = array(
                        $this->constants_text['PRODUCT_TYPE'] => $category
                    );
                    $product_details = array_merge($item, $product_type);
                    $item = $product_details;
                }
            }
            array_push($shopping_cart_items, $item);
        }
        return $shopping_cart_items;
    }

    /**
     * kfpwoo_map_payment_type
     * Return payment type
     * @param  mixed $payment_type
     * @return void
     */
    function kfpwoo_map_payment_type($payment_type)
    {
        $payment_type = strtolower($payment_type);
        $payment_type = str_replace(' ', '', $payment_type);
        $ptyp = $this->constants_text['DEFAULT_PAYMENT_TYPE'];
        foreach ($this->payment_type as $key => $value) {
            if ($key == $payment_type) {
                $ptyp = $value;
            }
        }
        return $ptyp;
    }

    /**
     * kfpwoo_get_coupon
     * Get coupon details
     * @return void
     */
    function kfpwoo_get_coupon()
    {
        $this->coupon_total = 0;
        $coupons = [];
        foreach ($this->order->get_coupons() as $code => $coupon) {
            $this->coupon_total++;
            $coupons_arr = (object)[
                $this->constants_text['NAME']     => $coupon->get_code(),
                $this->constants_text['AMOUNT']   => floatval($coupon->get_discount())
            ];
            array_push($coupons, $coupons_arr);
        }
        return $coupons;
    }
    /**
     * kfpwoo_get_order_properties
     * Get order items details
     * @return void
     */
    function kfpwoo_get_order_properties()
    {
        $order_properties = [];
        $exclude_details = ['_customer_user', '_payment_method', '_payment_method_title', '_customer_ip_address', '_customer_user_agent', '_billing_first_name', '_billing_last_name', '_billing_address_1', '_billing_address_2', '_billing_city', '_billing_state', '_billing_postcode', '_billing_country', '_billing_email', '_billing_phone', '_shipping_first_name', '_shipping_last_name', '_shipping_address_1', '_shipping_address_2', '_shipping_city', '_shipping_state', '_shipping_postcode', '_shipping_country', '_order_currency', '_order_total'];
        $order_meta = get_post_meta($this->order->get_id(), '', false);
        foreach ($order_meta as $meta_key => $meta_value) {
            if (!in_array($meta_key, $exclude_details)) {
                $item = (object)[
                    $this->constants_text['NAME']       => $meta_key,
                    $this->constants_text['VALUE']      => strval($meta_value[0]),
                ];
                array_push($order_properties, $item);
            }
        }
        return $order_properties;
    }
    /**
     * kfpwoo_create_json_payload
     * Create payload for order assessment ris api
     * @return void
     */
    function kfpwoo_create_json_payload()
    {
        $this->ptyp = $this->kfpwoo_map_payment_type($this->order->get_payment_method_title());
        $this->order_details = $this->order->get_data();
        if (is_user_logged_in()) {
            $this->user_data = get_userdata($this->order->get_user_id());
            $customer_details = (object)[
                $this->constants_text['CUSTOMER_ID']      =>  $this->order_details['customer_id'],
                $this->constants_text['USER_NAME']        =>  $this->user_data->display_name,
                $this->constants_text['EMAIL_ADDRESS']    =>  $this->user_data->user_email,
                $this->constants_text['FIRST_NAME']       =>  $this->user_data->first_name,
                $this->constants_text['LAST_NAME']        =>  $this->user_data->last_name,
                $this->constants_text['ACCOUNT_CREATION_DATE']   => strval(strtotime($this->user_data->user_registered)),
            ];
        } else {
            $customer_details = (object)[
                $this->constants_text['EMAIL_ADDRESS']    => ($this->order_details['billing']['email'] != "" ? $this->order_details['billing']['email'] : $this->constants_text['NO_EMAIL_KOUNT']),
                $this->constants_text['FIRST_NAME']       => ($this->order_details['billing']['first_name'] != "" ? $this->order_details['billing']['first_name'] : 'GUEST'),
                $this->constants_text['LAST_NAME']        => ($this->order_details['billing']['last_name'] != "" ? $this->order_details['billing']['last_name'] : 'GUEST')
            ];
        }
        foreach ($this->order->get_items('shipping') as $item_id => $item) {
            $shipping_method_id  =  $item->get_method_id();
            $shipping_method     = $item->get_method_title();
        }
        $this->payload = (object)[
            $this->constants_text['WOO_SITE']             =>  KFPWOO_Merchant_Settings::$payment_website,
            $this->constants_text['MODE']                 =>  $this->constants_text['PRE_AUTH_MODE'],
            $this->constants_text['PLUGIN_VERSION']       =>  $this->constants_text['PLUGIN_VERSION_VALUE'],
            $this->constants_text['WOO_PLATFORM_VERSION'] =>  $this->order_details['version'],
            $this->constants_text['KOUNT_MERCHANT_ID']    =>  KFPWOO_Merchant_Settings::$merchant_id,
            $this->constants_text['SOURCE']               => (wp_is_mobile() == true) ? 'mob' : 'web',
            $this->constants_text['CART_ID']              =>  $this->current_session_id,
            $this->constants_text['BROWSER_IP']           =>  $_SERVER['REMOTE_ADDR'],
            $this->constants_text['ORDER_NUMBER']         =>  strval($this->order->get_id()),
            $this->constants_text['CUSTOMER_DETAILS']     => $customer_details,
            $this->constants_text['BILLING_DETAILS']      => (object)[
                $this->constants_text['LINE1']            =>  $this->order_details['billing']['address_1'],
                $this->constants_text['LINE2']            =>  $this->order_details['billing']['address_2'],
                $this->constants_text['CITY']             =>  $this->order_details['billing']['city'],
                $this->constants_text['STATE']            =>  $this->order_details['billing']['state'],
                $this->constants_text['POSTAL_CODE']      =>  $this->order_details['billing']['postcode'],
                $this->constants_text['COUNTRY_CODE']     =>  $this->order_details['billing']['country'],
                $this->constants_text['PHONE_NUMBER']     =>  $this->order_details['billing']['phone']
            ],
            $this->constants_text['SHIPPING_DETAILS']           => (object)[
                $this->constants_text['LINE1']                  =>  $this->order_details['shipping']['address_1'],
                $this->constants_text['LINE2']                  =>  $this->order_details['shipping']['address_2'],
                $this->constants_text['CITY']                   =>  $this->order_details['shipping']['city'],
                $this->constants_text['STATE']                  =>  $this->order_details['shipping']['state'],
                $this->constants_text['POSTAL_CODE']            =>  $this->order_details['shipping']['postcode'],
                $this->constants_text['COUNTRY_CODE']           =>  $this->order_details['shipping']['country'],
                $this->constants_text['FIRST_NAME']             =>  $this->order_details['shipping']['first_name'],
                $this->constants_text['LAST_NAME']              =>  $this->order_details['shipping']['last_name'],
                $this->constants_text['EMAIL_ADDRESS']          =>  isset($this->order_details['shipping']['shipping_email']) ? $this->order_details['shipping']['shipping_email'] : $this->order_details['billing']['email'],
                $this->constants_text['PHONE_NUMBER']           =>  isset($this->order_details['shipping']['shipping_phone']) ? $this->order_details['shipping']['shipping_phone'] : $this->order_details['billing']['phone'],
                $this->constants_text['SHIPPING_METHOD_ID']     =>  $shipping_method_id,
                $this->constants_text['SHIPPING_METHOD']        =>  $shipping_method
            ],
            $this->constants_text['SHOPPING_CART']              =>  $this->kfpwoo_get_product_details(),
            $this->constants_text['TRANSACTION_DETAILS']        =>  (object)[
                $this->constants_text['PAYMENT_TRANSACTION_ID'] => $this->order->get_transaction_id(),
                $this->constants_text['PAYMENT_METHOD']         =>  $this->order->get_payment_method(),
                $this->constants_text['PAYMENT_METHOD_TITLE']   =>  $this->order->get_payment_method_title(),
                $this->constants_text['TYPE']                   =>  $this->constants_text['AUTHORIZATION'],
                $this->constants_text['PAYMENT_TYPE']           => ($this->ptyp == $this->constants_text['CARD']) ? 'NONE' : $this->ptyp,
                $this->constants_text['CURRENCY']               =>  $this->order_details['currency'],
                $this->constants_text['IS_TEST']                =>  true,
                $this->constants_text['TOTAL']                  =>  strval($this->order_details['total']),
                $this->constants_text['DISCOUNT_TOTAL']         =>  floatval($this->discount),
                $this->constants_text['COUPON_CODES']           =>  $this->kfpwoo_get_coupon(),
                $this->constants_text['COUPON_TOTAL']           =>  floatval($this->coupon_total),
                $this->constants_text['STATUS']                 =>  'A',

            ],
            $this->constants_text['ORDER_PROPERTIES']           =>  $this->kfpwoo_get_order_properties()
        ];
        $response = $this->kfpwoo_http_request($this->payload, 'POST');
        $this->kfpwoo_check_auto_status($response);
    }

    /**
     * kfpwoo_http_request
     * Calling RIS API
     * @return void
     */
    function kfpwoo_http_request($payload, $method)
    {
        /**API url */
        $ris_api = $this->kount_config['RIS_URL'] . $this->kount_config['RIS_ENDPOINT'];

        /**Request parameter */
        $request        = [
            $this->constants_text["API_URL"]       =>  $ris_api,
            $this->constants_text["API_KEY"]       =>  "Bearer " . KFPWOO_Merchant_Settings::$api_key,
            $this->constants_text["PAYLOAD"]       =>  wp_json_encode($payload),
            $this->constants_text["METHOD"]        =>  $method,
            $this->constants_text["HEADER"]        =>  "Authorization",
            $this->constants_text["CALL_FORM"]     =>  "RIS",
        ];

        $this->response = $this->call_api_obj->kfpwoo_call_api($request);
        if ($method == 'POST') {
            update_post_meta($this->order->get_id(), $this->constants_text['KOUNT_RIS_RESPONSE_FIELD'], $this->response->decision);
            $this->event_logging->kfpwoo_info_logs($this->constants_text['PRE_AUTH_LOGS'], $this->current_session_id, $this->constants_text['KOUNT_RIS_RESPONSE_FIELD'] . ":" . $this->response->decision);
        }
        /** handling response */
        if ($this->response) {
            $this->event_logging->kfpwoo_info_logs($this->constants_text['PRE_AUTH_LOGS'], $this->current_session_id, $this->constants_text['PRE_AUTH_SUCCESS']);
            return $this->response;
        } else {
            $this->event_logging->kfpwoo_error_logs($this->constants_text['PRE_AUTH_LOGS'], $this->current_session_id, $this->constants_text['PRE_AUTH_FAILED']);
            return $this->response;
        }
    }

    /**
     * kfpwoo_update_order_status
     * Update order status based on ris request response
     * @param  mixed $status
     * @return void
     */
    function kfpwoo_update_order_status($status, $is_redirect)
    {

        $this->order->update_status($status);
        $this->event_logging->kfpwoo_info_logs($this->constants_text['PRE_AUTH_LOGS'], $this->current_session_id, $this->constants_text['KOUNT_ORDER_STATUS_UPDATE'] . " : " . $status);
        wc_empty_cart();
        session_start();
        unset($_SESSION['KFPWOO_SESSION_ID']);
        if ($is_redirect) {
            wp_send_json(
                array(
                    'result'   => 'success',
                    'redirect' => $this->order->get_checkout_order_received_url()
                )
            );
        }
    }

    /**
     * kfpwoo_check_auto_status
     * Update order status based on auto field value
     * @param  mixed $response
     * @return void
     */
    function kfpwoo_check_auto_status($response)
    {
        session_start();
        $AUTO                           = $response->decision;
        $_SESSION['AUTO_FIELD_VALUE']   = $AUTO;
        $_SESSION['RIS_TRAN_ID']        = $response->transactionID;
        if ($AUTO == 'D') {
            //Decline Response
            $this->kfpwoo_update_order_status('cancelled', true);
            unset($_SESSION['KFPWOO_SESSION_ID']);
        }
    }

    /**
     * kfpwoo_check_order_status_change
     *
     * @param  mixed $order_id
     * @return void
     */
    public function kfpwoo_check_order_status_change($order_id)
    {
        if (!get_post_meta($order_id, 'kount_ris_is_assessment_sent', true)) {
            update_post_meta($order_id, 'kount_ris_is_assessment_sent', true);
            $this->kfpwoo_set_order($order_id);
            $order_status = $this->order->get_status();
            $auto_value = get_post_meta($order_id, 'kount_RIS_response', true);
            if ($auto_value == 'A' || $auto_value == 'R' || $auto_value == 'E') {
                switch ($order_status) {
                    case 'cancelled':
                        $this->auth = 'D';
                        break;
                    case 'on-hold':
                        $this->auth = 'A';
                        break;
                    case 'processing':
                        $this->auth = 'A';
                        break;
                    case 'failed':
                        $this->auth = 'D';
                        break;
                }
                if ($this->order->has_status(array('processing', 'pending')) && $this->auth == 'A' && ($auto_value == 'R' || $auto_value == 'E')) {
                    $this->kfpwoo_update_order_status('on-hold', false);
                }
                $this->kfpwoo_http_request($this->kfpwoo_payload_after_payment($this->auth), 'PUT');
                session_start();
                unset($_SESSION['KFPWOO_SESSION_ID']);
            }
        }
    }

    /**
     * kfpwoo_payload_after_payment
     *
     * @param  mixed $auth
     * @return void
     */
    public function kfpwoo_payload_after_payment($auth)
    {
        $parameters = (object)[
            $this->constants_text['WOO_SITE']                 => KFPWOO_Merchant_Settings::$payment_website,
            $this->constants_text['PLUGIN_VERSION']           => $this->constants_text['PLUGIN_VERSION_VALUE'],
            $this->constants_text['KOUNT_MERCHANT_ID']        => KFPWOO_Merchant_Settings::$merchant_id,
            $this->constants_text['KOUNT_TRANSACTION_ID_PUT'] => sanitize_text_field($_SESSION['RIS_TRAN_ID']),
            $this->constants_text['CART_ID']                  => $this->current_session_id,
            $this->constants_text['ORDER_NUMBER']             => strval($this->order->get_id()),
            $this->constants_text['TRANSACTION_DETAILS']      => (object)[
                $this->constants_text['STATUS'] => $auth,
            ]
        ];
        return $parameters;
    }
}
